@@ -1,6 +1,8 @@
 pub mod cleaner;
+pub mod command;
 pub mod filesystem;
 pub mod matcher;
-pub mod walker;
+pub mod models;
 #[cfg(test)]
-mod test_utils;
\ No newline at end of file
+mod test_utils;
+pub mod walker;
