@@ -1,8 +1,10 @@
-use std::{collections::HashSet, path::PathBuf, sync::Arc};
+use std::{collections::HashSet, path::PathBuf};
 
 use crate::{
-    filesystem::{FileInfo, FileSystem, SimpleFileKind},
-    matcher::Matcher,
+    filesystem::FileSystem,
+    matcher::{CleanStrategy, Matcher},
+    models::RemovalCandidate,
+    models::{FileInfo, SimpleFileKind},
 };
 use eyre::Report;
 use eyre::Result;
@@ -61,18 +63,36 @@ impl<FS: FileSystem, N: WalkNotifier> Walker<FS, N> {
         let mut entries = self.fs.list_files(&file)?;
 
         for matcher in &self.matchers {
-            entries = self.process_matcher(matcher, entries);
+            entries = self.process_matcher(file, matcher, entries);
         }
         entries.retain(|f| self.is_walkable(f));
         Ok(entries)
     }
 
-    fn process_matcher(&self, matcher: &Matcher, entries: Vec<FileInfo>) -> Vec<FileInfo> {
+    fn process_matcher(
+        &self,
+        work_dir: &FileInfo,
+        matcher: &Matcher,
+        entries: Vec<FileInfo>,
+    ) -> Vec<FileInfo> {
         if matcher.any_entry_match(&entries) {
-            let (mut to_remove, remaining) = matcher.find_files_to_remove(entries);
-            to_remove.retain(|p| !self.ignores.contains(&p.path));
-            self.notify_removal_candidates(matcher, to_remove);
-            remaining
+            match &matcher.clean_strategy {
+                CleanStrategy::Remove(pattern) => {
+                    let (mut to_remove, remaining) = pattern.find_files_to_remove(entries);
+                    to_remove.retain(|p| !self.ignores.contains(&p.path));
+                    self.notify_removal_candidates(matcher, to_remove);
+                    remaining
+                }
+                CleanStrategy::RunCommand(cmd) => {
+                    let candidate = RemovalCandidate::new_cmd(
+                        matcher.name.clone(),
+                        work_dir.clone(),
+                        cmd.clone(),
+                    );
+                    self.notifier.notify_candidate_for_removal(candidate);
+                    entries
+                }
+            }
         } else {
             entries
         }
@@ -95,23 +115,6 @@ impl<FS: FileSystem, N: WalkNotifier> Walker<FS, N> {
     }
 }
 
-#[derive(Debug)]
-pub struct RemovalCandidate {
-    pub matcher_name: Arc<str>,
-    pub file_info: FileInfo,
-    pub file_size: Option<u64>,
-}
-
-impl RemovalCandidate {
-    pub fn new(matcher_name: Arc<str>, file_info: FileInfo, file_size: Option<u64>) -> Self {
-        Self {
-            matcher_name,
-            file_info,
-            file_size,
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use std::{cell::RefCell, collections::HashSet, path::PathBuf, str::FromStr};
@@ -119,13 +122,15 @@ mod tests {
     use glob::Pattern;
 
     use crate::{
-        filesystem::{FileInfo, FileSystem},
+        filesystem::FileSystem,
         matcher::Matcher,
+        models::FileInfo,
         test_utils::{MockFS, MockFSNode},
         walker::Walker,
     };
 
-    use super::{RemovalCandidate, WalkNotifier};
+    use super::WalkNotifier;
+    use crate::models::{RemovalAction, RemovalCandidate};
 
     #[derive(Debug, Default)]
     struct VecWalkNotifier {
@@ -170,7 +175,7 @@ mod tests {
         let notifier = VecWalkNotifier::default();
         let walker = Walker::new(
             fs,
-            vec![Matcher::new(
+            vec![Matcher::with_remove_strategy(
                 "Cargo".into(),
                 Pattern::new("Cargo.toml")?,
                 Pattern::new("target")?,
@@ -186,7 +191,18 @@ mod tests {
         assert_eq!(1, to_remove.len());
         let c = to_remove.into_iter().next().unwrap();
         assert_eq!(c.matcher_name.as_ref(), "Cargo");
-        assert_eq!(c.file_info.path, PathBuf::from_str("/home/user/projectA/target").unwrap());
+
+        match c.action {
+            RemovalAction::Delete { file_info, .. } => {
+                assert_eq!(
+                    file_info.path,
+                    PathBuf::from_str("/home/user/projectA/target").unwrap()
+                )
+            }
+            RemovalAction::RunCommand { work_dir, command } => {
+                panic!("should be delete")
+            }
+        }
 
         Ok(())
     }
